@@ -38,12 +38,17 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 
 #include <libxfce4ui/libxfce4ui.h>
 #include <xfconf/xfconf.h>
-#include "xfwm4-tweaks-dialog_ui.h"
 
-static GdkNativeWindow opt_socket_id = 0;
+#include <common/xfwm-common.h>
+
+#include "xfwm4-tweaks-dialog_ui.h"
+#include "range-debouncer.h"
+
+static Window opt_socket_id = 0;
 static gboolean opt_version = FALSE;
 
 static const gchar *const modifier_list[] = {
@@ -168,6 +173,7 @@ wm_tweaks_dialog_configure_widgets (GtkBuilder *builder)
     GtkWidget *cycle_hidden_check = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_hidden_check"));
     GtkWidget *cycle_minimum_check = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_minimum_check"));
     GtkWidget *cycle_draw_frame = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_draw_frame"));
+    GtkWidget *cycle_raise = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_raise"));
     GtkWidget *cycle_tabwin_mode = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_tabwin_mode"));
 
     /* Focus tab */
@@ -205,7 +211,6 @@ wm_tweaks_dialog_configure_widgets (GtkBuilder *builder)
     GtkWidget *use_compositing_box = GTK_WIDGET (gtk_builder_get_object (builder, "use_compositing_box"));
 
     GtkWidget *unredirect_overlays_check = GTK_WIDGET (gtk_builder_get_object (builder, "unredirect_overlays_check"));
-    GtkWidget *sync_to_vblank_check = GTK_WIDGET (gtk_builder_get_object (builder, "sync_to_vblank_check"));
     GtkWidget *cycle_preview_check = GTK_WIDGET (gtk_builder_get_object (builder, "cycle_preview_check"));
     GtkWidget *show_frame_shadow_check = GTK_WIDGET (gtk_builder_get_object (builder, "show_frame_shadow_check"));
     GtkWidget *show_popup_shadow_check = GTK_WIDGET (gtk_builder_get_object (builder, "show_popup_shadow_check"));
@@ -313,6 +318,10 @@ wm_tweaks_dialog_configure_widgets (GtkBuilder *builder)
                             "/general/cycle_draw_frame",
                             G_TYPE_BOOLEAN,
                             (GObject *)cycle_draw_frame, "active");
+    xfconf_g_property_bind (xfwm4_channel,
+                            "/general/cycle_raise",
+                            G_TYPE_BOOLEAN,
+                            (GObject *)cycle_raise, "active");
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/cycle_tabwin_mode",
                             G_TYPE_INT,
@@ -399,14 +408,6 @@ wm_tweaks_dialog_configure_widgets (GtkBuilder *builder)
                             "/general/unredirect_overlays",
                             G_TYPE_BOOLEAN,
                             (GObject *)unredirect_overlays_check, "active");
-#ifdef HAVE_EPOXY
-    xfconf_g_property_bind (xfwm4_channel,
-                            "/general/sync_to_vblank",
-                            G_TYPE_BOOLEAN,
-                            (GObject *)sync_to_vblank_check, "active");
-#else
-    gtk_widget_hide (sync_to_vblank_check);
-#endif
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/cycle_preview",
                             G_TYPE_BOOLEAN,
@@ -426,23 +427,23 @@ wm_tweaks_dialog_configure_widgets (GtkBuilder *builder)
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/frame_opacity",
                             G_TYPE_INT,
-                            (GObject *) gtk_range_get_adjustment (GTK_RANGE (frame_opacity_scale)), "value");
+                            (GObject *) range_debouncer_bind (GTK_RANGE (frame_opacity_scale)), "value");
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/resize_opacity",
                             G_TYPE_INT,
-                            (GObject *) gtk_range_get_adjustment (GTK_RANGE (resize_opacity_scale)), "value");
+                            (GObject *) range_debouncer_bind (GTK_RANGE (resize_opacity_scale)), "value");
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/move_opacity",
                             G_TYPE_INT,
-                            (GObject *) gtk_range_get_adjustment (GTK_RANGE (move_opacity_scale)), "value");
+                            (GObject *) range_debouncer_bind (GTK_RANGE (move_opacity_scale)), "value");
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/inactive_opacity",
                             G_TYPE_INT,
-                            (GObject *) gtk_range_get_adjustment (GTK_RANGE (inactive_opacity_scale)), "value");
+                            (GObject *) range_debouncer_bind (GTK_RANGE (inactive_opacity_scale)), "value");
     xfconf_g_property_bind (xfwm4_channel,
                             "/general/popup_opacity",
                             G_TYPE_INT,
-                            (GObject *) gtk_range_get_adjustment (GTK_RANGE (popup_opacity_scale)), "value");
+                            (GObject *) range_debouncer_bind (GTK_RANGE (popup_opacity_scale)), "value");
 
     vbox = GTK_WIDGET (gtk_builder_get_object (builder, "main-vbox"));
     gtk_widget_show_all (vbox);
@@ -530,7 +531,7 @@ main (int argc, gchar **argv)
             g_signal_connect (dialog, "response", G_CALLBACK (wm_tweaks_dialog_response), NULL);
 
             /* To prevent the settings dialog to be saved in the session */
-            gdk_set_sm_client_id ("FAKE ID");
+            gdk_x11_set_sm_client_id ("FAKE ID");
 
             gtk_main ();
 
@@ -545,11 +546,11 @@ main (int argc, gchar **argv)
 
             /* Get plug child widget */
             plug_child = GTK_WIDGET (gtk_builder_get_object (builder, "plug-child"));
-            gtk_widget_reparent (plug_child, plug);
+            xfwm_widget_reparent (plug_child, plug);
             gtk_widget_show (plug_child);
 
             /* To prevent the settings dialog to be saved in the session */
-            gdk_set_sm_client_id ("FAKE ID");
+            gdk_x11_set_sm_client_id ("FAKE ID");
 
             /* Stop startup notification */
             gdk_notify_startup_complete ();
