@@ -67,8 +67,8 @@ handleXError (Display * dpy, XErrorEvent * err)
 
     XGetErrorText (dpy, err->error_code, buf, 63);
     fprintf (stderr, "XError: %s\n", buf);
-    fprintf (stderr, "==>  XID 0x%lx, Request %d, Error %d <==\n",
-              err->resourceid, err->request_code, err->error_code);
+    fprintf (stderr, "==>  XID 0x%lx, request code %d, serial %lu, error code %d, minor code %d <==\n",
+              err->resourceid, err->request_code, err->serial, err->error_code, err->minor_code);
 #endif
     return 0;
 }
@@ -257,28 +257,34 @@ myDisplayInit (GdkDisplay *gdisplay)
 
 #ifdef HAVE_XSYNC
     display->have_xsync = FALSE;
-
     display->xsync_error_base = 0;
     display->xsync_event_base = 0;
 
-    major = SYNC_MAJOR_VERSION;
-    minor = SYNC_MINOR_VERSION;
-
     if (XSyncQueryExtension (display->dpy,
                              &display->xsync_event_base,
-                             &display->xsync_error_base)
-         && XSyncInitialize (display->dpy,
-                             &major,
-                             &minor))
+                             &display->xsync_error_base))
     {
-        display->have_xsync = TRUE;
+         if (XSyncInitialize (display->dpy, &major, &minor))
+         {
+             if ((major > SYNC_MAJOR_VERSION) ||
+                 (major == SYNC_MAJOR_VERSION && minor >= SYNC_MINOR_VERSION))
+             {
+                 display->have_xsync = TRUE;
+                 XSyncSetPriority (display->dpy, None, 10);
+             }
+             else
+             {
+                 g_warning ("XSync extension too old (%i.%i).", major, minor);
+             }
+         }
+         else
+         {
+             g_warning ("Failed to initialize XSync extension.");
+         }
     }
     else
     {
         g_warning ("The display does not support the XSync extension.");
-        display->have_xsync = FALSE;
-        display->xsync_event_base = 0;
-        display->xsync_error_base = 0;
     }
 #endif /* HAVE_XSYNC */
 
@@ -781,8 +787,8 @@ myDisplayUpdateCurrentTime (DisplayInfo *display, XfwmEvent *event)
         case XFWM_EVENT_CROSSING:
             timestamp = (guint32) event->crossing.time;
             break;
-        case XFWM_EVENT_X:
-            xevent = event->meta.x;
+        case XFWM_EVENT_XEVENT:
+            xevent = event->meta.xevent;
 
             switch (xevent->type)
             {
@@ -1036,10 +1042,8 @@ gint
 myDisplayErrorTrapPop (DisplayInfo *display_info)
 {
 #if GTK_CHECK_VERSION(3, 22, 0)
-    gdk_display_flush (display_info->gdisplay);
     return gdk_x11_display_error_trap_pop (display_info->gdisplay);
 #else
-    gdk_flush ();
     return gdk_error_trap_pop ();
 #endif
 }
@@ -1073,4 +1077,3 @@ myDisplayGetKeymap (DisplayInfo *display_info)
     return gdk_keymap_get_default ();
 #endif
 }
-
