@@ -97,7 +97,8 @@ static void show_window_menu         (Client *c,
                                       gint px,
                                       gint py,
                                       guint button,
-                                      guint32 time);
+                                      guint32 time,
+                                      gboolean needscale);
 static gboolean show_popup_cb        (GtkWidget * widget,
                                       GdkEventButton * ev,
                                       gpointer data);
@@ -198,7 +199,7 @@ typeOfClick_event_filter (XfwmEvent *event, gpointer data)
         }
         status = EVENT_FILTER_STOP;
     }
-    else if ((event->meta.x->type == DestroyNotify) || (event->meta.x->type == UnmapNotify))
+    else if ((event->meta.xevent->type == DestroyNotify) || (event->meta.xevent->type == UnmapNotify))
     {
         if (event->meta.window == passdata->w)
         {
@@ -420,7 +421,7 @@ handleKeyPress (DisplayInfo *display_info, XfwmEventKey *event)
             case KEY_POPUP_MENU:
                 show_window_menu (c, frameExtentX (c) + frameLeft (c),
                                      frameExtentY (c) + frameTop (c),
-                                     Button1, event->time);
+                                     Button1, event->time, TRUE);
                 break;
             case KEY_FILL_WINDOW:
                 clientFill (c, CLIENT_FILL);
@@ -1086,9 +1087,11 @@ handleButtonPress (DisplayInfo *display_info, XfwmEventButton *event)
             }
             else
             {
+                myDisplayErrorTrapPush (display_info);
                 xfwm_device_ungrab (display_info->devices, &display_info->devices->pointer,
                                     display_info->dpy, event->time);
-                XSendEvent (display_info->dpy, screen_info->xfwm4_win, FALSE, SubstructureNotifyMask, event->meta.x);
+                XSendEvent (display_info->dpy, screen_info->xfwm4_win, FALSE, SubstructureNotifyMask, event->meta.xevent);
+                myDisplayErrorTrapPopIgnored (display_info);
             }
         }
     }
@@ -1108,14 +1111,16 @@ handleButtonRelease (DisplayInfo *display_info, XfwmEventButton *event)
 
     /* Get the screen structure from the root of the event */
     screen_info = myDisplayGetScreenFromRoot (display_info, event->root);
+    myDisplayErrorTrapPush (display_info);
     if (screen_info)
     {
         XSendEvent (display_info->dpy, screen_info->xfwm4_win, FALSE, SubstructureNotifyMask,
-                    (XEvent *) event->meta.x);
+                    (XEvent *) event->meta.xevent);
     }
 
     /* Release pending events */
     XAllowEvents (display_info->dpy, SyncPointer, CurrentTime);
+    myDisplayErrorTrapPopIgnored (display_info);
 
     return EVENT_FILTER_REMOVE;
 }
@@ -1374,7 +1379,9 @@ handleConfigureRequest (DisplayInfo *display_info, XConfigureRequestEvent * ev)
     else
     {
         TRACE ("unmanaged configure request for window 0x%lx", ev->window);
+        myDisplayErrorTrapPush (display_info);
         XConfigureWindow (display_info->dpy, ev->window, ev->value_mask, &wc);
+        myDisplayErrorTrapPopIgnored (display_info);
     }
 
     return EVENT_FILTER_REMOVE;
@@ -1978,7 +1985,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
         else if ((ev->message_type == display_info->atoms[GTK_SHOW_WINDOW_MENU]) && (ev->format == 32))
         {
             TRACE ("client \"%s\" (0x%lx) has received a GTK_SHOW_WINDOW_MENU event", c->name, c->window);
-            show_window_menu (c, (gint) ev->data.l[1], (gint) ev->data.l[2], Button3, (Time) myDisplayGetCurrentTime (display_info));
+            show_window_menu (c, (gint) ev->data.l[1], (gint) ev->data.l[2], Button3, (Time) myDisplayGetCurrentTime (display_info), TRUE);
         }
     }
     else
@@ -2202,7 +2209,7 @@ handleEvent (DisplayInfo *display_info, XfwmEvent *event)
 
     /* Update the display time */
     myDisplayUpdateCurrentTime (display_info, event);
-    sn_process_event (event->meta.x);
+    sn_process_event (event->meta.xevent);
 
     switch (event->meta.type)
     {
@@ -2239,59 +2246,59 @@ handleEvent (DisplayInfo *display_info, XfwmEvent *event)
                 status = handleLeaveNotify (display_info, &event->crossing);
             }
             break;
-        case XFWM_EVENT_X:
-            switch (event->meta.x->type)
+        case XFWM_EVENT_XEVENT:
+            switch (event->meta.xevent->type)
             {
                 case DestroyNotify:
-                    status = handleDestroyNotify (display_info, (XDestroyWindowEvent *) event->meta.x);
+                    status = handleDestroyNotify (display_info, (XDestroyWindowEvent *) event->meta.xevent);
                     break;
                 case UnmapNotify:
-                    status = handleUnmapNotify (display_info, (XUnmapEvent *) event->meta.x);
+                    status = handleUnmapNotify (display_info, (XUnmapEvent *) event->meta.xevent);
                     break;
                 case MapRequest:
-                    status = handleMapRequest (display_info, (XMapRequestEvent *) event->meta.x);
+                    status = handleMapRequest (display_info, (XMapRequestEvent *) event->meta.xevent);
                     break;
                 case MapNotify:
-                    status = handleMapNotify (display_info, (XMapEvent *) event->meta.x);
+                    status = handleMapNotify (display_info, (XMapEvent *) event->meta.xevent);
                     break;
                 case ConfigureNotify:
-                    status = handleConfigureNotify (display_info, (XConfigureEvent *) event->meta.x);
+                    status = handleConfigureNotify (display_info, (XConfigureEvent *) event->meta.xevent);
                     break;
                 case ConfigureRequest:
-                    status = handleConfigureRequest (display_info, (XConfigureRequestEvent *) event->meta.x);
+                    status = handleConfigureRequest (display_info, (XConfigureRequestEvent *) event->meta.xevent);
                     break;
                 case FocusIn:
-                    status = handleFocusIn (display_info, (XFocusChangeEvent *) event->meta.x);
+                    status = handleFocusIn (display_info, (XFocusChangeEvent *) event->meta.xevent);
                     break;
                 case FocusOut:
-                    status = handleFocusOut (display_info, (XFocusChangeEvent *) event->meta.x);
+                    status = handleFocusOut (display_info, (XFocusChangeEvent *) event->meta.xevent);
                     break;
                 case PropertyNotify:
-                    status = handlePropertyNotify (display_info, (XPropertyEvent *) event->meta.x);
+                    status = handlePropertyNotify (display_info, (XPropertyEvent *) event->meta.xevent);
                     break;
                 case ClientMessage:
-                    status = handleClientMessage (display_info, (XClientMessageEvent *) event->meta.x);
+                    status = handleClientMessage (display_info, (XClientMessageEvent *) event->meta.xevent);
                     break;
                 case SelectionClear:
-                    status = handleSelectionClear (display_info, (XSelectionClearEvent *) event->meta.x);
+                    status = handleSelectionClear (display_info, (XSelectionClearEvent *) event->meta.xevent);
                     break;
                 case ColormapNotify:
-                    handleColormapNotify (display_info, (XColormapEvent *) event->meta.x);
+                    handleColormapNotify (display_info, (XColormapEvent *) event->meta.xevent);
                     break;
                 case ReparentNotify:
-                    status = handleReparentNotify (display_info, (XReparentEvent *) event->meta.x);
+                    status = handleReparentNotify (display_info, (XReparentEvent *) event->meta.xevent);
                     break;
                 default:
                     if ((display_info->have_shape) &&
-                            (event->meta.x->type == display_info->shape_event_base))
+                            (event->meta.xevent->type == display_info->shape_event_base))
                     {
-                        status = handleShape (display_info, (XShapeEvent *) event->meta.x);
+                        status = handleShape (display_info, (XShapeEvent *) event->meta.xevent);
                     }
 #ifdef HAVE_XSYNC
                     if ((display_info->have_xsync) &&
-                            (event->meta.x->type == (display_info->xsync_event_base + XSyncAlarmNotify)))
+                            (event->meta.xevent->type == (display_info->xsync_event_base + XSyncAlarmNotify)))
                     {
-                        status = handleXSyncAlarmNotify (display_info, (XSyncAlarmNotifyEvent *) event->meta.x);
+                        status = handleXSyncAlarmNotify (display_info, (XSyncAlarmNotifyEvent *) event->meta.xevent);
                     }
 #endif /* HAVE_XSYNC */
                     break;
@@ -2316,7 +2323,7 @@ handleEvent (DisplayInfo *display_info, XfwmEvent *event)
         }
     }
 
-    compositorHandleEvent (display_info, event->meta.x);
+    compositorHandleEvent (display_info, event->meta.xevent);
 
     return status;
 }
@@ -2448,7 +2455,7 @@ initMenuEventWin (void)
 }
 
 static void
-show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
+show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp, gboolean needscale)
 {
     ScreenInfo *screen_info;
     DisplayInfo *display_info;
@@ -2457,6 +2464,7 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
     MenuOp insensitive;
     gboolean is_transient;
     gint x, y;
+    gint scale = 1;
 
     TRACE ("coords (%d,%d)", px, py);
 
@@ -2473,15 +2481,27 @@ show_window_menu (Client *c, gint px, gint py, guint button, guint32 timestamp)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     is_transient = clientIsValidTransientOrModal (c);
+#if GTK_CHECK_VERSION(3, 22, 0)
+    scale = gdk_window_get_scale_factor (gdk_screen_get_root_window (screen_info->gscr));
+#endif
 
     x = px;
     y = py;
 
+    if (needscale) {
+        x /= scale;
+        y /= scale;
+    }
+
     c->button_status[MENU_BUTTON] = BUTTON_STATE_PRESSED;
     frameQueueDraw (c, FALSE);
     if (CLIENT_HAS_FRAME (c))
     {
-        y = c->y;
+        x = px;
+        y = c->y / scale;
+        if (needscale) {
+            x /= scale;
+        }
     }
     ops = MENU_OP_DELETE | MENU_OP_MINIMIZE_ALL | MENU_OP_WORKSPACES | MENU_OP_MOVE | MENU_OP_RESIZE;
     insensitive = 0;
@@ -2670,7 +2690,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
 {
     TRACE ("entering");
 
-    show_window_menu ((Client *) data, (gint) ev->x_root, (gint) ev->y_root, ev->button, ev->time);
+    show_window_menu ((Client *) data, (gint) ev->x_root, (gint) ev->y_root, ev->button, ev->time, FALSE);
 
     return (TRUE);
 }
