@@ -525,7 +525,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         clientSetNetState (c);
         restacked = clientAdjustFullscreenLayer (c, TRUE);
 
-        if (!restacked && screen_info->params->click_to_focus)
+        if (!restacked && screen_info->params->raise_on_focus)
         {
             clientRaise (c, None);
             clientSetLastRaise (c);
@@ -582,6 +582,7 @@ clientSetFocus (ScreenInfo *screen_info, Client *c, guint32 timestamp, unsigned
              * When shaded, the client window is unmapped, so it can not be focused.
              * Instead, we focus the frame that is still mapped.
              */
+            myDisplayErrorTrapPush (screen_info->display_info);
             if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
             {
                 XSetInputFocus (myScreenGetXDisplay (screen_info), c->frame, RevertToPointerRoot, timestamp);
@@ -590,6 +591,12 @@ clientSetFocus (ScreenInfo *screen_info, Client *c, guint32 timestamp, unsigned
             {
                 XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
             }
+            if (myDisplayErrorTrapPop (screen_info->display_info) != Success)
+            {
+                client_focus = NULL;
+                clientFocusNone (screen_info, c2, timestamp);
+                clientClearDelayedFocus ();
+            }
         }
         else if (flags & FOCUS_TRANSITION)
         {
@@ -606,7 +613,7 @@ clientSetFocus (ScreenInfo *screen_info, Client *c, guint32 timestamp, unsigned
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
         {
             pending_focus = c;
-            sendClientMessage (c->screen_info, c->window, WM_TAKE_FOCUS, timestamp);
+            sendClientMessage (screen_info, c->window, WM_TAKE_FOCUS, timestamp);
         }
     }
     else
@@ -702,16 +709,18 @@ clientGrabMouseButton (Client * c)
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     screen_info = c->screen_info;
+    myDisplayErrorTrapPush (screen_info->display_info);
     if (screen_info->params->raise_with_any_button)
     {
-        grabButton (c->screen_info->display_info->devices, clientGetXDisplay (c),
+        grabButton (screen_info->display_info->devices, clientGetXDisplay (c),
                     AnyButton, AnyModifier, c->window);
     }
     else
     {
-        grabButton (c->screen_info->display_info->devices, clientGetXDisplay (c),
+        grabButton (screen_info->display_info->devices, clientGetXDisplay (c),
                     Button1, AnyModifier, c->window);
     }
+    myDisplayErrorTrapPopIgnored (screen_info->display_info);
 }
 
 void
@@ -720,10 +729,12 @@ clientUngrabMouseButton (Client * c)
     g_return_if_fail (c != NULL);
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
+    myDisplayErrorTrapPush (c->screen_info->display_info);
     ungrabButton (c->screen_info->display_info->devices, clientGetXDisplay (c),
                   AnyButton, AnyModifier, c->window);
     /* We've ungrabbed way too much, so regrab the regular buttons/modifiers */
     clientGrabButtons (c);
+    myDisplayErrorTrapPopIgnored (c->screen_info->display_info);
 }
 
 void
