@@ -45,275 +45,31 @@ typedef struct
     xfwmPixmap pm_sides[SIDE_COUNT];
 } FramePixmap;
 
-int
-frameDecorationLeft (ScreenInfo *screen_info)
+static int
+frameDecorationBorderTop (ScreenInfo *screen_info)
 {
     TRACE ("entering");
 
     g_return_val_if_fail (screen_info != NULL, 0);
-    return screen_info->sides[SIDE_LEFT][ACTIVE].width;
+    return screen_info->params->frame_border_top;
 }
 
-int
-frameDecorationRight (ScreenInfo *screen_info)
-{
-    TRACE ("entering");
-
-    g_return_val_if_fail (screen_info != NULL, 0);
-    return screen_info->sides[SIDE_RIGHT][ACTIVE].width;
-}
-
-int
-frameDecorationTop (ScreenInfo *screen_info)
-{
-    TRACE ("entering");
-
-    g_return_val_if_fail (screen_info != NULL, 0);
-    return screen_info->title[TITLE_3][ACTIVE].height;
-}
-
-int
-frameDecorationBottom (ScreenInfo *screen_info)
-{
-    TRACE ("entering");
-
-    g_return_val_if_fail (screen_info != NULL, 0);
-    return screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
-}
-
-int
-frameLeft (Client * c)
+static int
+frameBorderTop (Client * c)
 {
     g_return_val_if_fail (c != NULL, 0);
     TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
 
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-            || !(c->screen_info->params->borderless_maximize)))
+        && FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+        && (c->screen_info->params->borderless_maximize))
     {
-        return c->screen_info->sides[SIDE_LEFT][ACTIVE].width;
+        return frameDecorationBorderTop (c->screen_info);
     }
     return 0;
 }
 
-int
-frameRight (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-            || !(c->screen_info->params->borderless_maximize)))
-    {
-        return c->screen_info->sides[SIDE_RIGHT][ACTIVE].width;
-    }
-    return 0;
-}
-
-int
-frameTop (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (CLIENT_HAS_FRAME (c))
-    {
-        return c->screen_info->title[TITLE_3][ACTIVE].height;
-    }
-    return 0;
-}
-
-int
-frameBottom (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-            || !(c->screen_info->params->borderless_maximize)))
-    {
-        return c->screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
-    }
-    return 0;
-}
-
-int
-frameX (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-            || !(c->screen_info->params->borderless_maximize)))
-    {
-        return c->x - frameLeft (c);
-    }
-    return c->x;
-}
-
-int
-frameY (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
-    {
-        return c->y - frameTop (c);
-    }
-    return c->y;
-}
-
-int
-frameWidth (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
-        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
-            || !(c->screen_info->params->borderless_maximize)))
-    {
-        return c->width + frameLeft (c) + frameRight (c);
-    }
-    return c->width;
-}
-
-int
-frameHeight (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-        && FLAG_TEST (c->flags, CLIENT_FLAG_SHADED)
-        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
-    {
-        return frameTop (c) + frameBottom (c);
-    }
-    else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
-             && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
-    {
-        return c->height + frameTop (c) + frameBottom (c);
-    }
-    return c->height;
-}
-
-int
-frameExtentLeft (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return -c->frame_extents[SIDE_LEFT];
-    }
-    return frameLeft(c);
-}
-
-int
-frameExtentRight (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return -c->frame_extents[SIDE_RIGHT];
-    }
-    return frameRight(c);
-}
-
-int
-frameExtentTop (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return -c->frame_extents[SIDE_TOP];
-    }
-    return frameTop(c);
-}
-
-int
-frameExtentBottom (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return -c->frame_extents[SIDE_BOTTOM];
-    }
-    return frameBottom(c);
-}
-
-int
-frameExtentX (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return c->x + c->frame_extents[SIDE_LEFT];
-    }
-    return frameX(c);
-}
-
-int
-frameExtentY (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return c->y + c->frame_extents[SIDE_TOP];
-    }
-    return frameY(c);
-}
-
-int
-frameExtentWidth (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return MAX (0, c->width - c->frame_extents[SIDE_LEFT]
-                                - c->frame_extents[SIDE_RIGHT]);
-    }
-    return frameWidth(c);
-}
-
-int
-frameExtentHeight (Client * c)
-{
-    g_return_val_if_fail (c != NULL, 0);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
-    {
-        return MAX (0, c->height - c->frame_extents[SIDE_TOP]
-                                 - c->frame_extents[SIDE_BOTTOM]);
-    }
-    return frameHeight(c);
-}
-
 static int
 frameTopLeftWidth (Client * c, int state)
 {
@@ -377,7 +133,7 @@ frameFillTitlePixmap (Client * c, int state, int part, int x, int w, int h, xfwm
     {
         xfwmPixmapFill (&screen_info->title[part][state], top_pm, x, 0, w, h);
     }
-    xfwmPixmapFill (&screen_info->title[part][state], title_pm, x, 0, w, frameTop (c));
+    xfwmPixmapFill (&screen_info->title[part][state], title_pm, x, 0, w, frameDecorationTop(screen_info));
 }
 
 static void
@@ -443,6 +199,10 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     layout = gtk_widget_create_pango_layout (myScreenGetGtkWidget (screen_info), c->name);
     pango_layout_set_font_description (layout, myScreenGetFontDescription (screen_info));
     pango_layout_set_auto_dir (layout, FALSE);
+    if (screen_info->pango_attr_list != NULL)
+    {
+        pango_layout_set_attributes (layout, screen_info->pango_attr_list);
+    }
     pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
     title_height = screen_info->font_height;
@@ -453,19 +213,23 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
          */
         title_height = logical_rect.height;
     }
-    title_y = voffset + (frameTop (c) - title_height) / 2;
-    if (title_y + title_height > frameTop (c))
+    title_y = voffset + (frameDecorationTop(screen_info) - title_height) / 2;
+    if (title_y + title_height > frameDecorationTop(screen_info))
     {
-        title_y = MAX (0, frameTop (c) - title_height);
+        title_y = MAX (0, frameDecorationTop(screen_info) - title_height);
     }
 
     if (!xfwmPixmapNone(&screen_info->top[3][ACTIVE]))
     {
         top_height = screen_info->top[3][ACTIVE].height;
     }
+    else if (frameDecorationBorderTop(c->screen_info) > 0)
+    {
+        top_height = frameDecorationBorderTop(c->screen_info);
+    }
     else
     {
-        top_height = frameTop (c) / 10 + 1;
+        top_height = frameDecorationTop(screen_info) / 10 + 1;
         if (top_height > title_y - 1)
         {
              top_height = MAX (title_y - 1, 0);
@@ -533,7 +297,7 @@ frameCreateTitlePixmap (Client * c, int state, int left, int right, xfwmPixmap *
     }
 
     xfwmPixmapCreate (screen_info, top_pm, width, top_height);
-    xfwmPixmapCreate (screen_info, title_pm, width, frameTop (c));
+    xfwmPixmapCreate (screen_info, title_pm, width, frameDecorationTop(screen_info));
 
     surface = xfwmPixmapCreateSurface (title_pm, FALSE);
     cr = cairo_create (surface);
@@ -946,39 +710,6 @@ frameSetShape (Client * c, int state, FramePixmap * frame_pix, int button_x[BUTT
     XShapeCombineShape (display_info->dpy, c->frame, ShapeBounding, 0, 0, screen_info->shape_win, ShapeBounding, ShapeSet);
 }
 
-void
-frameSetShapeInput (Client * c)
-{
-    ScreenInfo *screen_info;
-    DisplayInfo *display_info;
-
-    g_return_if_fail (c != NULL);
-    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
-
-    screen_info = c->screen_info;
-    display_info = screen_info->display_info;
-
-    if (!myDisplayHaveShapeInput(display_info))
-    {
-        return;
-    }
-
-    if (screen_info->shape_win == None)
-    {
-        screen_info->shape_win = XCreateSimpleWindow (display_info->dpy, screen_info->xroot, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
-    }
-    else
-    {
-        XResizeWindow (display_info->dpy, screen_info->shape_win, frameWidth (c), frameHeight (c));
-    }
-
-    /* Set Input shape when using XShape extension 1.1 and later */
-    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, 0, 0, c->frame, ShapeBounding, ShapeSet);
-    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeBounding, ShapeSubtract);
-    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeInput, ShapeUnion);
-    XShapeCombineShape(display_info->dpy, c->frame, ShapeInput, 0, 0, screen_info->shape_win, ShapeInput, ShapeSet);
-}
-
 static void
 frameDrawWin (Client * c)
 {
@@ -1098,7 +829,7 @@ frameDrawWin (Client * c)
                         xfwmWindowSetBG (&c->buttons[button], my_pixmap);
                     }
                     xfwmWindowShow (&c->buttons[button], x,
-                        (frameTop (c) - screen_info->buttons[button][state].height + 1) / 2,
+                        ((frameDecorationTop(screen_info) - screen_info->buttons[button][state].height + 1) / 2) - frameBorderTop (c),
                         screen_info->buttons[button][state].width,
                         screen_info->buttons[button][state].height, TRUE);
                     button_x[button] = x;
@@ -1135,7 +866,7 @@ frameDrawWin (Client * c)
                     x = x - screen_info->buttons[button][state].width -
                         screen_info->params->button_spacing;
                     xfwmWindowShow (&c->buttons[button], x,
-                        (frameTop (c) - screen_info->buttons[button][state].height + 1) / 2,
+                        ((frameDecorationTop(screen_info) - screen_info->buttons[button][state].height + 1) / 2) - frameBorderTop (c),
                         screen_info->buttons[button][state].width,
                         screen_info->buttons[button][state].height, TRUE);
                     button_x[button] = x;
@@ -1168,8 +899,8 @@ frameDrawWin (Client * c)
         frameCreateTitlePixmap (c, state, left, right, &frame_pix.pm_title, &frame_pix.pm_sides[SIDE_TOP]);
         xfwmWindowSetBG (&c->title, &frame_pix.pm_title);
         xfwmWindowShow (&c->title,
-            frameTopLeftWidth (c, state), 0, top_width,
-            frameTop (c), (requires_clearing | width_changed));
+            frameTopLeftWidth (c, state), 0 - frameBorderTop (c), top_width,
+            frameDecorationTop(screen_info), (requires_clearing | width_changed));
 
         /* Corners are never resized, we need to update them separately */
         if (requires_clearing)
@@ -1333,6 +1064,275 @@ update_frame_idle_cb (gpointer data)
     return (FALSE);
 }
 
+int
+frameDecorationLeft (ScreenInfo *screen_info)
+{
+    TRACE ("entering");
+
+    g_return_val_if_fail (screen_info != NULL, 0);
+    return screen_info->sides[SIDE_LEFT][ACTIVE].width;
+}
+
+int
+frameDecorationRight (ScreenInfo *screen_info)
+{
+    TRACE ("entering");
+
+    g_return_val_if_fail (screen_info != NULL, 0);
+    return screen_info->sides[SIDE_RIGHT][ACTIVE].width;
+}
+
+int
+frameDecorationTop (ScreenInfo *screen_info)
+{
+    TRACE ("entering");
+
+    g_return_val_if_fail (screen_info != NULL, 0);
+    return screen_info->title[TITLE_3][ACTIVE].height;
+}
+
+int
+frameDecorationBottom (ScreenInfo *screen_info)
+{
+    TRACE ("entering");
+
+    g_return_val_if_fail (screen_info != NULL, 0);
+    return screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
+}
+
+int
+frameLeft (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
+    {
+        return c->screen_info->sides[SIDE_LEFT][ACTIVE].width;
+    }
+    return 0;
+}
+
+int
+frameRight (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
+    {
+        return c->screen_info->sides[SIDE_RIGHT][ACTIVE].width;
+    }
+    return 0;
+}
+
+int
+frameTop (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (CLIENT_HAS_FRAME (c))
+    {
+        return frameDecorationTop(c->screen_info) - frameBorderTop (c);
+    }
+    return 0;
+}
+
+int
+frameBottom (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
+    {
+        return c->screen_info->sides[SIDE_BOTTOM][ACTIVE].height;
+    }
+    return 0;
+}
+
+int
+frameX (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
+    {
+        return c->x - frameLeft (c);
+    }
+    return c->x;
+}
+
+int
+frameY (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+         && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+    {
+        return c->y - frameTop (c);
+    }
+    return c->y;
+}
+
+int
+frameWidth (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN)
+        && (!FLAG_TEST_ALL (c->flags, CLIENT_FLAG_MAXIMIZED)
+            || !(c->screen_info->params->borderless_maximize)))
+    {
+        return c->width + frameLeft (c) + frameRight (c);
+    }
+    return c->width;
+}
+
+int
+frameHeight (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+        && FLAG_TEST (c->flags, CLIENT_FLAG_SHADED)
+        && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+    {
+        return frameTop (c) + frameBottom (c);
+    }
+    else if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_HAS_BORDER)
+             && !FLAG_TEST (c->flags, CLIENT_FLAG_FULLSCREEN))
+    {
+        return c->height + frameTop (c) + frameBottom (c);
+    }
+    return c->height;
+}
+
+int
+frameExtentLeft (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return -c->frame_extents[SIDE_LEFT];
+    }
+    return frameLeft(c);
+}
+
+int
+frameExtentRight (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return -c->frame_extents[SIDE_RIGHT];
+    }
+    return frameRight(c);
+}
+
+int
+frameExtentTop (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return -c->frame_extents[SIDE_TOP];
+    }
+    return frameTop(c);
+}
+
+int
+frameExtentBottom (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return -c->frame_extents[SIDE_BOTTOM];
+    }
+    return frameBottom(c);
+}
+
+int
+frameExtentX (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return c->x + c->frame_extents[SIDE_LEFT];
+    }
+    return frameX(c);
+}
+
+int
+frameExtentY (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return c->y + c->frame_extents[SIDE_TOP];
+    }
+    return frameY(c);
+}
+
+int
+frameExtentWidth (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return MAX (0, c->width - c->frame_extents[SIDE_LEFT]
+                                - c->frame_extents[SIDE_RIGHT]);
+    }
+    return frameWidth(c);
+}
+
+int
+frameExtentHeight (Client * c)
+{
+    g_return_val_if_fail (c != NULL, 0);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_FRAME_EXTENTS))
+    {
+        return MAX (0, c->height - c->frame_extents[SIDE_TOP]
+                                 - c->frame_extents[SIDE_BOTTOM]);
+    }
+    return frameHeight(c);
+}
+
 void
 frameClearQueueDraw (Client * c)
 {
@@ -1382,3 +1382,35 @@ frameQueueDraw (Client * c, gboolean clear_all)
     }
 }
 
+void
+frameSetShapeInput (Client * c)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+
+    g_return_if_fail (c != NULL);
+    TRACE ("client \"%s\" (0x%lx)", c->name, c->window);
+
+    screen_info = c->screen_info;
+    display_info = screen_info->display_info;
+
+    if (!myDisplayHaveShapeInput(display_info))
+    {
+        return;
+    }
+
+    if (screen_info->shape_win == None)
+    {
+        screen_info->shape_win = XCreateSimpleWindow (display_info->dpy, screen_info->xroot, 0, 0, frameWidth (c), frameHeight (c), 0, 0, 0);
+    }
+    else
+    {
+        XResizeWindow (display_info->dpy, screen_info->shape_win, frameWidth (c), frameHeight (c));
+    }
+
+    /* Set Input shape when using XShape extension 1.1 and later */
+    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, 0, 0, c->frame, ShapeBounding, ShapeSet);
+    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeBounding, ShapeSubtract);
+    XShapeCombineShape(display_info->dpy, screen_info->shape_win, ShapeInput, frameLeft (c), frameTop (c), c->window, ShapeInput, ShapeUnion);
+    XShapeCombineShape(display_info->dpy, c->frame, ShapeInput, 0, 0, screen_info->shape_win, ShapeInput, ShapeSet);
+}
