@@ -306,6 +306,7 @@ myScreenInit (DisplayInfo *display_info, GdkScreen *gscr, unsigned long event_ma
 
     screen_info->font_height = 0;
     screen_info->font_desc = NULL;
+    screen_info->pango_attr_list = NULL;
     screen_info->box_gc = None;
 
     for (i = 0; i < SIDE_COUNT; i++)
@@ -391,6 +392,11 @@ myScreenClose (ScreenInfo *screen_info)
         screen_info->monitors_index = NULL;
     }
 
+    if (screen_info->pango_attr_list)
+    {
+        pango_attr_list_unref (screen_info->pango_attr_list);
+    }
+
     return (screen_info);
 }
 
@@ -434,11 +440,13 @@ myScreenGrabKeyboard (ScreenInfo *screen_info, guint event_mask, guint32 timesta
     grab = TRUE;
     if (screen_info->key_grabs == 0)
     {
+        myDisplayErrorTrapPush (screen_info->display_info);
         grab = xfwm_device_grab (screen_info->display_info->devices,
                                  &screen_info->display_info->devices->keyboard,
                                  myScreenGetXDisplay (screen_info), screen_info->xroot,
                                  TRUE, event_mask, GrabModeAsync, screen_info->xroot,
                                  None, (Time) timestamp);
+        myDisplayErrorTrapPopIgnored (screen_info->display_info);
     }
     screen_info->key_grabs++;
     TRACE ("global key grabs %i", screen_info->key_grabs);
@@ -458,11 +466,13 @@ myScreenGrabPointer (ScreenInfo *screen_info, gboolean owner_events,
     grab = TRUE;
     if (screen_info->pointer_grabs == 0)
     {
+        myDisplayErrorTrapPush (screen_info->display_info);
         grab = xfwm_device_grab (screen_info->display_info->devices,
                                  &screen_info->display_info->devices->pointer,
                                  myScreenGetXDisplay (screen_info), screen_info->xroot,
                                  owner_events, event_mask, GrabModeAsync, screen_info->xroot,
                                  cursor, (Time) timestamp);
+        myDisplayErrorTrapPopIgnored (screen_info->display_info);
     }
     screen_info->pointer_grabs++;
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
@@ -482,6 +492,7 @@ myScreenChangeGrabPointer (ScreenInfo *screen_info, gboolean owner_events,
     grab = FALSE;
     if (screen_info->pointer_grabs > 0)
     {
+        myDisplayErrorTrapPush (screen_info->display_info);
         if (screen_info->display_info->devices->pointer.xi2_device == None)
         {
             grab = (XChangeActivePointerGrab (myScreenGetXDisplay (screen_info),
@@ -495,6 +506,7 @@ myScreenChangeGrabPointer (ScreenInfo *screen_info, gboolean owner_events,
                                      owner_events, event_mask, GrabModeAsync, screen_info->xroot,
                                      cursor, (Time) timestamp);
         }
+        myDisplayErrorTrapPopIgnored (screen_info->display_info);
     }
 
     return grab;
@@ -513,9 +525,11 @@ myScreenUngrabKeyboard (ScreenInfo *screen_info, guint32 timestamp)
     }
     if (screen_info->key_grabs == 0)
     {
+        myDisplayErrorTrapPush (screen_info->display_info);
         xfwm_device_ungrab (screen_info->display_info->devices,
                             &screen_info->display_info->devices->keyboard,
                             myScreenGetXDisplay (screen_info), (Time) timestamp);
+        myDisplayErrorTrapPopIgnored (screen_info->display_info);
     }
     TRACE ("global key grabs %i", screen_info->key_grabs);
 
@@ -535,9 +549,11 @@ myScreenUngrabPointer (ScreenInfo *screen_info, guint32 timestamp)
     }
     if (screen_info->pointer_grabs == 0)
     {
+        myDisplayErrorTrapPush (screen_info->display_info);
         xfwm_device_ungrab (screen_info->display_info->devices,
                             &screen_info->display_info->devices->pointer,
                             myScreenGetXDisplay (screen_info), (Time) timestamp);
+        myDisplayErrorTrapPopIgnored (screen_info->display_info);
     }
     TRACE ("global pointer grabs %i", screen_info->pointer_grabs);
 
@@ -596,7 +612,7 @@ myScreenGetModifierPressed (ScreenInfo *screen_info)
 {
     int rx, ry;
 
-    return (int) getMouseXY (screen_info, screen_info->xroot, &rx, &ry);
+    return (int) getMouseXY (screen_info, &rx, &ry);
 }
 
 Client *
@@ -642,28 +658,16 @@ myScreenComputeSize (ScreenInfo *screen_info)
 
     for (i = 0; i < num_monitors; i++)
     {
-        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor, TRUE);
         width = MAX (monitor.x + monitor.width, width);
         height = MAX (monitor.y + monitor.height, height);
     }
 
-    screen_info->logical_width = WidthOfScreen (screen_info->xscreen);
-    screen_info->logical_height = HeightOfScreen (screen_info->xscreen);
-    if ((width != screen_info->logical_width) || (height != screen_info->logical_height))
-    {
-        g_warning ("output size (%dx%d) and logical screen size (%dx%d) do not match",
-                   width, height, screen_info->logical_width, screen_info->logical_height);
-    }
-
     /* If we failed to compute the size, use whatever xlib reports */
-    if (width == 0)
+    if (width == 0 || height == 0)
     {
-        width = screen_info->logical_width;
-    }
-
-    if (height == 0)
-    {
-        height = screen_info->logical_height;
+        width = WidthOfScreen (screen_info->xscreen);
+        height = HeightOfScreen (screen_info->xscreen);
     }
 
     changed = ((screen_info->width != width) | (screen_info->height != height));
@@ -717,11 +721,11 @@ myScreenRebuildMonitorIndex (ScreenInfo *screen_info)
     num_monitors = xfwm_get_n_monitors (screen_info->gscr);
     for (i = 0; i < num_monitors; i++)
     {
-        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, i, &monitor, TRUE);
         cloned = FALSE;
         for (j = 0; j < (gint) screen_info->monitors_index->len; j++)
         {
-            xfwm_get_monitor_geometry (screen_info->gscr, j, &previous);
+            xfwm_get_monitor_geometry (screen_info->gscr, j, &previous, TRUE);
             if ((previous.x == monitor.x) && (previous.y == monitor.y))
             {
                 cloned = TRUE;
@@ -784,7 +788,7 @@ myScreenFindMonitorAtPoint (ScreenInfo *screen_info, gint x, gint y, GdkRectangl
         gint monitor_index;
 
         monitor_index = myScreenGetMonitorIndex (screen_info, i);
-        xfwm_get_monitor_geometry (screen_info->gscr, monitor_index, &monitor);
+        xfwm_get_monitor_geometry (screen_info->gscr, monitor_index, &monitor, TRUE);
 
         if ((x >= monitor.x) && (x < (monitor.x + monitor.width)) &&
             (y >= monitor.y) && (y < (monitor.y + monitor.height)))
@@ -868,7 +872,10 @@ myScreenUpdateFontHeight (ScreenInfo *screen_info)
     PangoFontDescription *desc;
     PangoContext *context;
     PangoFontMetrics *metrics;
+    PangoAttribute *attr;
     GtkWidget *widget;
+    gint font_height;
+    gint scale;
 
     g_return_val_if_fail (screen_info != NULL, FALSE);
 
@@ -879,11 +886,29 @@ myScreenUpdateFontHeight (ScreenInfo *screen_info)
     if (desc != NULL && context != NULL)
     {
         metrics = pango_context_get_metrics (context, desc, NULL);
-        screen_info->font_height =
+        scale = gtk_widget_get_scale_factor (widget);
+        font_height =
                  PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
-                               pango_font_metrics_get_descent (metrics));
+                               pango_font_metrics_get_descent (metrics)) * scale;
         pango_font_metrics_unref (metrics);
 
+        if (font_height != screen_info->font_height)
+        {
+            screen_info->font_height = font_height;
+
+            if (screen_info->pango_attr_list != NULL)
+            {
+                pango_attr_list_unref (screen_info->pango_attr_list);
+                screen_info->pango_attr_list = NULL;
+            }
+            if (scale != 1)
+            {
+                screen_info->pango_attr_list = pango_attr_list_new ();
+                attr = pango_attr_scale_new (scale);
+                pango_attr_list_insert (screen_info->pango_attr_list, attr);
+            }
+        }
+
         return TRUE;
     }
 
