@@ -76,6 +76,8 @@ struct _gaussian_conv {
 };
 typedef struct _gaussian_conv gaussian_conv;
 
+#endif /* HAVE_COMPOSITOR */
+
 typedef enum
 {
     VBLANK_OFF = 0,
@@ -84,7 +86,6 @@ typedef enum
     VBLANK_GLX,
     VBLANK_ERROR,
 } vblankMode;
-#endif /* HAVE_COMPOSITOR */
 
 struct _ScreenInfo
 {
@@ -116,6 +117,7 @@ struct _ScreenInfo
     /* Title font */
     gint font_height;
     PangoFontDescription *font_desc;
+    PangoAttrList *pango_attr_list;
 
     /* Screen data */
     Colormap cmap;
@@ -164,6 +166,10 @@ struct _ScreenInfo
     /* show desktop flag */
     gboolean show_desktop;
 
+    /* tabwin css provider */
+    gboolean tabwin_provider_ready;
+    GtkCssProvider *tabwin_provider;
+
 #ifdef ENABLE_KDE_SYSTRAY_PROXY
     /* There can be one systray per screen */
     Atom net_system_tray_selection;
