@@ -128,6 +128,7 @@ sn_collect_timed_out_foreach (void *element, void *data)
     SnStartupSequence *sequence;
     time_t tv_sec;
     suseconds_t tv_usec;
+    long l_sec, l_usec;
     double elapsed;
 
     g_return_if_fail (data != NULL);
@@ -135,7 +136,8 @@ sn_collect_timed_out_foreach (void *element, void *data)
 
     sequence = element;
     ctod = (CollectTimedOutData *) data;
-    sn_startup_sequence_get_last_active_time (sequence, &tv_sec, &tv_usec);
+    sn_startup_sequence_get_last_active_time (sequence, &l_sec, &l_usec);
+    tv_sec = l_sec; tv_usec = l_sec;
 
     elapsed =
         ((((double) ctod->now.tv_sec - tv_sec) * G_USEC_PER_SEC +
